@@ -60,10 +60,21 @@ def make_node(
 
     # ONNX weight input
     kernel_shape = graph_node.attrs.get('kernel_shape', [])
-    input_weights = get_weights_constant_or_variable(
-        const_or_var=graph_node.inputs[1],
-        kernel_size=len(kernel_shape),
-    )
+    try:
+        input_weights = get_weights_constant_or_variable(
+            const_or_var=graph_node.inputs[1] \
+                if graph_node.i(1).op != 'DequantizeLinear' \
+                    else tf.transpose(
+                            tf_layers_dict[graph_node.inputs[1].name]['tf_node'],
+                            perm=[0]+[len(tf_layers_dict[graph_node.inputs[1].name]['tf_node'].shape)-1]+[i for i in range(1, len(tf_layers_dict[graph_node.inputs[1].name]['tf_node'].shape)-1)]
+                        ),
+            kernel_size=len(kernel_shape),
+        )
+    except Exception as ex:
+        input_weights = get_weights_constant_or_variable(
+            const_or_var=graph_node.inputs[1],
+            kernel_size=len(kernel_shape),
+        )
     input_weights = tf_layers_dict[input_weights.name]['tf_node'] \
         if isinstance(input_weights, gs.Variable) else input_weights
 
@@ -164,10 +175,13 @@ def make_node(
                 )
                 sys.exit(1)
             onnx_graph = kwargs['onnx_graph']
-            convtranspose_output = dummy_onnx_inference(
-                onnx_graph=onnx_graph,
-                output_names=[graph_node_output.name],
-            )[0]
+            use_cuda = bool(kwargs['use_cuda'])
+            convtranspose_output = \
+                dummy_onnx_inference(
+                    onnx_graph=onnx_graph,
+                    output_names=[graph_node_output.name],
+                    use_cuda=use_cuda,
+                )[0]
             onnx_output_shape = list(convtranspose_output.shape)
             tf_output_shape = []
             for idx in range(input_tensor_rank):
