@@ -1,4 +1,3 @@
-import sys
 from typing import List, Dict
 import random
 random.seed(0)
@@ -16,46 +15,131 @@ from onnx2tf.utils.common_functions import (
     pre_process_transpose,
     post_process_transpose,
 )
-from onnx2tf.utils.colors import Color
 from tensorflow.python.keras.layers import Layer
 
 
-class Affine(tf.keras.layers.Layer):
+class Elu(Layer):
+    def __init__(self, alpha: float, beta: float):
+        super(Elu, self).__init__()
+
+    def call(self, x):
+        return tf.nn.elu(x)
+
+class HardSigmoid(Layer):
+    def __init__(self, alpha: float, beta: float):
+        super(HardSigmoid, self).__init__()
+        self.alpha = 0.2
+        self.beta = 0.5
+        self.alpha = tf.convert_to_tensor(alpha) \
+            if self.alpha != alpha else tf.convert_to_tensor(self.alpha)
+        self.beta = tf.convert_to_tensor(beta) \
+            if self.beta != beta else tf.convert_to_tensor(self.beta)
+
+    def call(self, x):
+        # https://github.com/onnx/onnx/blob/main/docs/Changelog.md#hardsigmoid-6
+        # max(0, min(1, alpha * x + beta))
+        return tf.maximum(0.0, tf.minimum(1.0, self.alpha * x + self.beta))
+
+class LeakyReLU(Layer):
+    def __init__(self, alpha: float, beta: float):
+        super(LeakyReLU, self).__init__()
+        self.alpha = 0.01
+        self.alpha = tf.convert_to_tensor(alpha) \
+            if self.alpha != alpha else tf.convert_to_tensor(self.alpha)
+
+    def call(self, x):
+        # https://github.com/onnx/onnx/blob/main/docs/Changelog.md#leakyrelu-16
+        return tf.nn.leaky_relu(x, alpha=self.alpha)
+
+class ReLU(Layer):
+    def __init__(self, alpha: float, beta: float):
+        super(ReLU, self).__init__()
+
+    def call(self, x):
+        return tf.nn.relu(x)
+
+class Sigmoid(Layer):
+    def __init__(self, alpha: float, beta: float):
+        super(Sigmoid, self).__init__()
+
+    def call(self, x):
+        return tf.sigmoid(x)
+
+class Softsign(Layer):
+    def __init__(self, alpha: float, beta: float):
+        super(Softsign, self).__init__()
+
+    def call(self, x):
+        return tf.nn.softsign(x)
+
+class Softplus(Layer):
+    def __init__(self, alpha: float, beta: float):
+        super(Softplus, self).__init__()
+
+    def call(self, x):
+        return tf.nn.softplus(x)
+
+class Tanh(Layer):
+    def __init__(self, alpha: float, beta: float):
+        super(Tanh, self).__init__()
+
+    def call(self, x):
+        return tf.tanh(x)
+
+class ThresholdedReLU(Layer):
+    def __init__(self, alpha: float, beta: float):
+        super(ThresholdedReLU, self).__init__()
+        self.alpha = 1.0
+        self.alpha = tf.convert_to_tensor(alpha) \
+            if self.alpha != alpha else tf.convert_to_tensor(self.alpha)
+
+    def call(self, x):
+        return tf.keras.layers.ThresholdedReLU(theta=self.alpha)(x)
+
+class Affine(Layer):
     def __init__(self, alpha: float, beta: float):
         super(Affine, self).__init__()
-        self.alpha = tf.convert_to_tensor(alpha)
-        self.beta = tf.convert_to_tensor(beta)
+        self.alpha = 1.0
+        self.beta = 0.0
+        self.alpha = tf.convert_to_tensor(alpha) \
+            if self.alpha != alpha else tf.convert_to_tensor(self.alpha)
+        self.beta = tf.convert_to_tensor(beta) \
+            if self.beta != beta else tf.convert_to_tensor(self.beta)
 
     def call(self, x):
         # https://github.com/onnx/onnx/blob/main/docs/Changelog.md#lstm-14
         # alpha*x + beta
         return self.alpha * x + self.beta
 
-class ScaledTanh(tf.keras.layers.Layer):
+class ScaledTanh(Layer):
     def __init__(self, alpha: float, beta: float):
         super(ScaledTanh, self).__init__()
-        self.alpha = tf.convert_to_tensor(alpha)
-        self.beta = tf.convert_to_tensor(beta)
+        self.alpha = 1.0
+        self.beta = 1.0
+        self.alpha = tf.convert_to_tensor(alpha) \
+            if self.alpha != alpha else tf.convert_to_tensor(self.alpha)
+        self.beta = tf.convert_to_tensor(beta) \
+            if self.beta != beta else tf.convert_to_tensor(self.beta)
 
     def call(self, x):
         # https://github.com/onnx/onnx/blob/main/docs/Changelog.md#lstm-7
         # alpha*Tanh(beta*x)
-        return self.alpha * tf.nn.tanh(self.beta*x)
+        return self.alpha * tf.nn.tanh(self.beta * x)
 
 
 # {'activateion_name': [tf_activation, default_alpha, default_beta]}
 ONNX_ACTIVATION_MAPPING: Dict[str, List] = {
-    "Elu": [tf.nn.elu, 1.0, 0.0],
-    "HardSigmoid": [tf.keras.backend.hard_sigmoid, 0.2, 0.5],
-    "LeakyRelu": [tf.nn.leaky_relu, 0.01, 0.0],
-    "Relu": [tf.nn.relu, 1.0, 0.0],
-    "Sigmoid": [tf.sigmoid, 1.0, 0.0],
-    "Softsign": [tf.nn.softsign, 1.0, 0.0],
-    "Softplus": [tf.nn.softplus, 1.0, 0.0],
-    "Tanh": [tf.tanh, 1.0, 0.0],
-    "ThresholdedRelu": [tf.keras.layers.ThresholdedReLU, 1.0, 0.0],
+    "Elu": [Elu, 1.0, 0.0],
+    "HardSigmoid": [HardSigmoid, 0.2, 0.5],
+    "LeakyRelu": [LeakyReLU, 0.01, 0.0],
+    "Relu": [ReLU, 1.0, 0.0],
+    "Sigmoid": [Sigmoid, 1.0, 0.0],
+    "Softsign": [Softsign, 1.0, 0.0],
+    "Softplus": [Softplus, 1.0, 0.0],
+    "Tanh": [Tanh, 1.0, 0.0],
+    "ThresholdedRelu": [ThresholdedReLU, 1.0, 0.0],
     "Affine": [Affine, 1.0, 0.0],
-    "ScaledTanh": [ScaledTanh, 1.0, 1.0]
+    "ScaledTanh": [ScaledTanh, 1.0, 1.0],
 }
 
 
@@ -72,6 +156,11 @@ class CustomLSTMCell(tf.keras.layers.AbstractRNNCell):
         bias_f,
         bias_c,
         bias_o,
+        peephole_i,
+        peephole_f,
+        peephole_o,
+        clip,
+        input_forget,
         is_bidirectional,
         go_backwards,
         **kwargs
@@ -87,18 +176,13 @@ class CustomLSTMCell(tf.keras.layers.AbstractRNNCell):
         self.bf = bias_f
         self.bc = bias_c
         self.bo = bias_o
+        self.peephole_i = peephole_i
+        self.peephole_f = peephole_f
+        self.peephole_o = peephole_o
+        self.clip = clip
+        self.input_forget = input_forget
         self.is_bidirectional = is_bidirectional
         self.go_backwards = go_backwards
-        self.dense_i = tf.keras.layers.Dense(
-            units=4 * self.hidden_size,
-            kernel_initializer=tf.keras.initializers.constant(self.kernel),
-            use_bias=False,
-        )
-        self.dense_h = tf.keras.layers.Dense(
-            units=4 * self.hidden_size,
-            kernel_initializer=tf.keras.initializers.constant(self.recurrent_kernel),
-            use_bias=False,
-        )
 
     @property
     def state_size(self):
@@ -112,27 +196,75 @@ class CustomLSTMCell(tf.keras.layers.AbstractRNNCell):
             it = f( Xt*(Wi^T) + Ht-1*(Ri^T) + Pi (.) Ct-1 + Wbi + Rbi )
             ft = f( Xt*(Wf^T) + Ht-1*(Rf^T) + Pf (.) Ct-1 + Wbf + Rbf )
             ct = g( Xt*(Wc^T) + Ht-1*(Rc^T) + Wbc + Rbc )
-            ot = f( Xt*(Wo^T) + Ht-1*(Ro^T) + Po (.) Ct + Wbo + Rbo )
 
             Ct = ft (.) Ct-1 + it (.) ct
+
+            ot = f( Xt*(Wo^T) + Ht-1*(Ro^T) + Po (.) Ct + Wbo + Rbo )
+
             Ht = ot (.) h( Ct )
         """
-
-        # TODO:
-        # 1. Pが考慮されていない
         h_prev, c_prev = states
-        gates = self.dense_i(inputs) + self.dense_h(h_prev)
+        gates = tf.matmul(inputs, self.kernel) + tf.matmul(h_prev, self.recurrent_kernel)
         i, f, c_candidate, o = tf.split(gates, num_or_size_splits=4, axis=-1)
+        if self.peephole_i is not None:
+            i = i + tf.matmul(self.peephole_i, c_prev)
+        if self.peephole_f is not None:
+            f = f + tf.matmul(self.peephole_f, c_prev)
 
         offsetidx = 3 if self.is_bidirectional and self.go_backwards else 0
 
-        i = self.activations[0 + offsetidx](i * self.activation_alphas[0 + offsetidx] + self.activation_betas[0 + offsetidx] + self.bi)
-        f = self.activations[0 + offsetidx](f * self.activation_alphas[0 + offsetidx] + self.activation_betas[0 + offsetidx] + self.bf)
-        c_candidate = self.activations[1 + offsetidx](c_candidate * self.activation_alphas[1 + offsetidx] + self.activation_betas[1 + offsetidx] + self.bc)
-        o = self.activations[0 + offsetidx](o * self.activation_alphas[0 + offsetidx] + self.activation_betas[0 + offsetidx] + self.bo)
+        if not self.clip:
+            i = self.activations[0 + offsetidx](i + self.bi)
+            if not self.input_forget:
+                f = self.activations[0 + offsetidx](f + self.bf)
+            else:
+                f = 1.0 - i
+            c_candidate = self.activations[1 + offsetidx](c_candidate + self.bc)
+        else:
+            i = self.activations[0 + offsetidx](
+                tf.clip_by_value(
+                    i + self.bi,
+                    clip_value_min=-self.clip,
+                    clip_value_max=self.clip,
+                )
+            )
+            if not self.input_forget:
+                f = self.activations[0 + offsetidx](
+                    tf.clip_by_value(
+                        f + self.bf,
+                        clip_value_min=-self.clip,
+                        clip_value_max=self.clip,
+                    )
+                )
+            else:
+                f = 1.0 - i
+            c_candidate = self.activations[1 + offsetidx](
+                tf.clip_by_value(
+                    c_candidate + self.bc,
+                    clip_value_min=-self.clip,
+                    clip_value_max=self.clip,
+                )
+            )
 
         c = f * c_prev + i * c_candidate
-        h = o * self.activations[2 + offsetidx](c * self.activation_alphas[2 + offsetidx] + self.activation_betas[2 + offsetidx])
+
+        if self.peephole_o is None:
+            o = o + self.bo
+        else:
+            o = o + tf.matmul(self.peephole_o, c) + self.bo
+
+        if not self.clip:
+            o = self.activations[0 + offsetidx](o)
+        else:
+            o = self.activations[0 + offsetidx](
+                tf.clip_by_value(
+                    o,
+                    clip_value_min=-self.clip,
+                    clip_value_max=self.clip,
+                )
+            )
+
+        h = o * self.activations[2 + offsetidx](c)
 
         return h, [h, c]
 
@@ -150,8 +282,14 @@ class CustomLSTM(Layer):
         bias_f,
         bias_c,
         bias_o,
+        peephole_i,
+        peephole_f,
+        peephole_o,
+        clip,
+        input_forget,
         is_bidirectional,
         go_backwards,
+        enable_rnn_unroll,
         return_sequences=True,
         **kwargs
     ):
@@ -165,10 +303,16 @@ class CustomLSTM(Layer):
         self.return_sequences = return_sequences
         self.is_bidirectional = is_bidirectional
         self.go_backwards = go_backwards
+        self.enable_rnn_unroll = enable_rnn_unroll
         self.bias_i = bias_i
         self.bias_f = bias_f
         self.bias_c = bias_c
         self.bias_o = bias_o
+        self.peephole_i = peephole_i
+        self.peephole_f = peephole_f
+        self.peephole_o = peephole_o
+        self.clip = clip
+        self.input_forget = input_forget
 
         self.cell = CustomLSTMCell(
             self.hidden_size,
@@ -181,6 +325,11 @@ class CustomLSTM(Layer):
             self.bias_f,
             self.bias_c,
             self.bias_o,
+            self.peephole_i,
+            self.peephole_f,
+            self.peephole_o,
+            self.clip,
+            self.input_forget,
             self.is_bidirectional,
             self.go_backwards,
         )
@@ -189,6 +338,7 @@ class CustomLSTM(Layer):
             return_sequences=self.return_sequences,
             go_backwards=self.go_backwards,
             return_state=True,
+            unroll=self.enable_rnn_unroll,
         )
 
     def call(self, inputs, initial_state=None):
@@ -205,9 +355,7 @@ def make_node(
     tf_layers_dict: dict,
     **kwargs: dict,
 ):
-    """[WIP][TODO] LSTM
-    Need to implement CustomLSTMCell and CustomLSTM to handle activation functions for f, g, and h
-    https://zenn.dev/pinto0309/scraps/430cea62b1eb9d
+    """LSTM
 
     https://github.com/PINTO0309/onnx2tf/issues/198
     test onnx file: https://s3.ap-northeast-2.wasabisys.com/temp-models/onnx2tf_198/text_recognition_CRNN_EN_2021sep.onnx
@@ -358,15 +506,6 @@ def make_node(
     P = tf_layers_dict[graph_node_input_8.name]['tf_node'] \
         if isinstance(graph_node_input_8, gs.Variable) and graph_node_input_8.name != '' else graph_node_input_8
 
-    if isinstance(P, np.ndarray) and np.sum(P) != 0.0:
-        print(
-            f'{Color.RED}ERROR:{Color.RESET} ' +
-            f'The process for the case where Peepholes is set to a value greater than zero has not yet been implemented. ' +
-            f'https://zenn.dev/pinto0309/scraps/430cea62b1eb9d ' +
-            f'P.shape: {P.shape}'
-        )
-        sys.exit(1)
-
     # Always three or more present if specified
     #   forward, reverse: 3 items
     #   bidirectional: 6 items
@@ -386,14 +525,6 @@ def make_node(
     tf_activation_betas: List = None
 
     clip: float =  graph_node.attrs.get('clip', None)
-    if clip is not None:
-        print(
-            f'{Color.RED}ERROR:{Color.RESET} ' +
-            f'clip is currently not implemented. ' +
-            f'clip: {clip}'
-        )
-        sys.exit(1)
-
     direction: str =  graph_node.attrs.get('direction', 'forward')
     if len(activations) == 0:
         # https://github.com/onnx/onnx/blob/main/docs/Changelog.md#LSTM-14
@@ -404,78 +535,66 @@ def make_node(
             'Tanh',    # h (Output Gate)
         ]
         tf_activations = [
-            ONNX_ACTIVATION_MAPPING[default_activations[0]][0], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[1]][0], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[2]][0], # h (Output Gate)
+            # f (Oblivion Gate)
+            ONNX_ACTIVATION_MAPPING[default_activations[0]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[default_activations[0]][1], beta=ONNX_ACTIVATION_MAPPING[default_activations[0]][2]
+            ),
+            # g (Input Gate)
+            ONNX_ACTIVATION_MAPPING[default_activations[1]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[default_activations[1]][1], beta=ONNX_ACTIVATION_MAPPING[default_activations[1]][2]
+            ),
+            # h (Output Gate)
+            ONNX_ACTIVATION_MAPPING[default_activations[2]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[default_activations[2]][1], beta=ONNX_ACTIVATION_MAPPING[default_activations[2]][2]
+            ),
         ]
         tf_activations = tf_activations + [
-            ONNX_ACTIVATION_MAPPING[default_activations[0]][0], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[1]][0], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[2]][0], # h (Output Gate)
+            # f (Oblivion Gate)
+            ONNX_ACTIVATION_MAPPING[default_activations[0]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[default_activations[0]][1], beta=ONNX_ACTIVATION_MAPPING[default_activations[0]][2]
+            ),
+            # g (Input Gate)
+            ONNX_ACTIVATION_MAPPING[default_activations[1]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[default_activations[1]][1], beta=ONNX_ACTIVATION_MAPPING[default_activations[1]][2]
+            ),
+            # h (Output Gate)
+            ONNX_ACTIVATION_MAPPING[default_activations[2]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[default_activations[2]][1], beta=ONNX_ACTIVATION_MAPPING[default_activations[2]][2]
+            ),
         ] if direction == 'bidirectional' else tf_activations
-        tf_activation_alphas = [
-            ONNX_ACTIVATION_MAPPING[default_activations[0]][1], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[1]][1], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[2]][1], # h (Output Gate)
-        ]
-        tf_activation_alphas = tf_activation_alphas + [
-            ONNX_ACTIVATION_MAPPING[default_activations[0]][1], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[1]][1], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[2]][1], # h (Output Gate)
-        ] if direction == 'bidirectional' else tf_activation_alphas
-        tf_activation_betas = [
-            ONNX_ACTIVATION_MAPPING[default_activations[0]][2], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[1]][2], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[2]][2], # h (Output Gate)
-        ]
-        tf_activation_betas = tf_activation_betas + [
-            ONNX_ACTIVATION_MAPPING[default_activations[0]][2], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[1]][2], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[default_activations[2]][2], # h (Output Gate)
-        ] if direction == 'bidirectional' else tf_activation_betas
+
     else:
         tf_activations = [
-            ONNX_ACTIVATION_MAPPING[activations[0]][0], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[activations[1]][0], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[activations[2]][0], # h (Output Gate)
+            # f (Oblivion Gate)
+            ONNX_ACTIVATION_MAPPING[activations[0]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[activations[0]][1], beta=ONNX_ACTIVATION_MAPPING[activations[0]][2]
+            ),
+            # g (Input Gate)
+            ONNX_ACTIVATION_MAPPING[activations[1]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[activations[1]][1], beta=ONNX_ACTIVATION_MAPPING[activations[1]][2]
+            ),
+            # h (Output Gate)
+            ONNX_ACTIVATION_MAPPING[activations[2]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[activations[2]][1], beta=ONNX_ACTIVATION_MAPPING[activations[2]][2]
+            ),
         ]
         tf_activations = tf_activations + [
-            ONNX_ACTIVATION_MAPPING[activations[3]][0], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[activations[4]][0], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[activations[5]][0], # h (Output Gate)
+            # f (Oblivion Gate)
+            ONNX_ACTIVATION_MAPPING[activations[3]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[activations[3]][1], beta=ONNX_ACTIVATION_MAPPING[activations[3]][2]
+            ),
+            # g (Input Gate)
+            ONNX_ACTIVATION_MAPPING[activations[4]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[activations[4]][1], beta=ONNX_ACTIVATION_MAPPING[activations[4]][2]
+            ),
+            # h (Output Gate)
+            ONNX_ACTIVATION_MAPPING[activations[5]][0](
+                alpha=ONNX_ACTIVATION_MAPPING[activations[5]][1], beta=ONNX_ACTIVATION_MAPPING[activations[5]][2]
+            ),
         ] if direction == 'bidirectional' else tf_activations
-        tf_activation_alphas = [
-            ONNX_ACTIVATION_MAPPING[activations[0]][1], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[activations[1]][1], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[activations[2]][1], # h (Output Gate)
-        ]
-        tf_activation_alphas = tf_activation_alphas + [
-            ONNX_ACTIVATION_MAPPING[activations[3]][1], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[activations[4]][1], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[activations[5]][1], # h (Output Gate)
-        ] if direction == 'bidirectional' else tf_activation_alphas
-        tf_activation_betas = [
-            ONNX_ACTIVATION_MAPPING[activations[0]][2], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[activations[1]][2], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[activations[2]][2], # h (Output Gate)
-        ]
-        tf_activation_betas = tf_activation_betas + [
-            ONNX_ACTIVATION_MAPPING[activations[3]][2], # f (Oblivion Gate)
-            ONNX_ACTIVATION_MAPPING[activations[4]][2], # g (Input Gate)
-            ONNX_ACTIVATION_MAPPING[activations[5]][2], # h (Output Gate)
-        ] if direction == 'bidirectional' else tf_activation_betas
 
     hidden_size: int =  graph_node.attrs.get('hidden_size', 1)
     input_forget: bool = bool(graph_node.attrs.get('input_forget', 0))
-
-    if input_forget:
-        print(
-            f'{Color.RED}ERROR:{Color.RESET} ' +
-            f'input_forget = 1 is currently not implemented. ' +
-            f'input_forget: {input_forget}'
-        )
-        sys.exit(1)
-
     layout: int = graph_node.attrs.get('layout', 0)
 
     # Need transpose for batchwise, X
@@ -502,6 +621,7 @@ def make_node(
         shape3 = graph_node_output3.shape
         dtype3 = graph_node_output3.dtype
 
+    enable_rnn_unroll: bool = kwargs['enable_rnn_unroll']
 
     # Preserving Graph Structure (Dict)
     tf_layers_dict[graph_node_output1.name] = {
@@ -668,14 +788,22 @@ def make_node(
 
     # LSTM layer
     if direction == 'forward':
-        forward_weight = tf.reshape(W[0], shape=[4, hidden_size, input_size]) # TensorShape([4, 256, 512])
-        forward_recurrence_weight = tf.reshape(R[0], shape=[4, hidden_size, hidden_size]) # TensorShape([4, 256, 256])
-        forward_bias_W = tf.reshape(B[0][:4*hidden_size], shape=[4, hidden_size]) # TensorShape([4, 256])
-        forward_bias_R = tf.reshape(B[0][4*hidden_size:4*hidden_size*2], shape=[4, hidden_size]) # TensorShape([4, 256])
+        forward_weight = tf.reshape(tf.convert_to_tensor(W[0]), shape=[4, hidden_size, input_size]) # TensorShape([4, 256, 512])
+        forward_recurrence_weight = tf.reshape(tf.convert_to_tensor(R[0]), shape=[4, hidden_size, hidden_size]) # TensorShape([4, 256, 256])
+        forward_bias_W = tf.reshape(tf.convert_to_tensor(B[0][:4*hidden_size]), shape=[4, hidden_size]) # TensorShape([4, 256])
+        forward_bias_R = tf.reshape(tf.convert_to_tensor(B[0][4*hidden_size:4*hidden_size*2]), shape=[4, hidden_size]) # TensorShape([4, 256])
         forward_bias = forward_bias_W + forward_bias_R
+        forward_peepholes = None
+        if P is not None:
+            forward_peepholes = tf.reshape(tf.convert_to_tensor(P[0]), shape=[3, hidden_size])
         fW_i, fW_o, fW_f, fW_c = tf.split(value=forward_weight, num_or_size_splits=4, axis=0) # (1, 256, 512)
         fR_i, fR_o, fR_f, fR_c = tf.split(value=forward_recurrence_weight, num_or_size_splits=4, axis=0) # (1, 256, 256)
         fB_i, fB_o, fB_f, fB_c = tf.split(value=forward_bias, num_or_size_splits=4, axis=0) # (1, 256)
+        fP_i = None
+        fP_o = None
+        fP_f = None
+        if forward_peepholes is not None:
+            fP_i, fP_o, fP_f = tf.split(value=forward_peepholes, num_or_size_splits=3, axis=0) # (1, 256)
         forward_kernel = tf.reshape(tf.transpose(tf.concat([fW_i, fW_f, fW_c, fW_o], axis=1), perm=[2, 0, 1]), shape=[input_size, -1]) # (1, 256*4, 512) -> (1, 1024, 512) -> (512, 1, 1024) -> (512, 1024)
         forward_recurrent_kernel = tf.reshape(tf.transpose(tf.concat([fR_i, fR_f, fR_c, fR_o], axis=1), perm=[2, 0, 1]), shape=[hidden_size, -1]) # (1, 256*4, 256) -> (256, 1, 1024) -> (256, 1024)
         # forward
@@ -690,8 +818,14 @@ def make_node(
             bias_f=fB_f, # (1, 256)
             bias_c=fB_c, # (1, 256)
             bias_o=fB_o, # (1, 256)
+            peephole_i=fP_i, # (1, 256)
+            peephole_f=fP_f, # (1, 256)
+            peephole_o=fP_o, # (1, 256)
+            clip=clip,
+            input_forget=input_forget,
             is_bidirectional=False,
             go_backwards=False,
+            enable_rnn_unroll=enable_rnn_unroll,
         )
         output, hidden_state, cell_state = forward_lstm(X, initial_state=forward_initial_state)
         output = tf.expand_dims(output, axis=1)
@@ -704,9 +838,17 @@ def make_node(
         reverse_bias_W = tf.reshape(tf.convert_to_tensor(B[0][:4*hidden_size]), shape=[4, hidden_size]) # TensorShape([4, 256])
         reverse_bias_R = tf.reshape(tf.convert_to_tensor(B[0][4*hidden_size:4*hidden_size*2]), shape=[4, hidden_size]) # TensorShape([4, 256])
         reverse_bias = reverse_bias_W + reverse_bias_R
+        reverse_peepholes = None
+        if P is not None:
+            reverse_peepholes = tf.reshape(tf.convert_to_tensor(P[0]), shape=[3, hidden_size])
         rW_i, rW_o, rW_f, rW_c = tf.split(value=reverse_weight, num_or_size_splits=4, axis=0)
         rR_i, rR_o, rR_f, rR_c = tf.split(value=reverse_recurrence_weight, num_or_size_splits=4, axis=0)
         rB_i, rB_o, rB_f, rB_c = tf.split(value=reverse_bias, num_or_size_splits=4, axis=0)
+        rP_i = None
+        rP_o = None
+        rP_f = None
+        if reverse_peepholes is not None:
+            rP_i, rP_o, rP_f = tf.split(value=reverse_peepholes, num_or_size_splits=3, axis=0) # (1, 256)
         reverse_kernel = tf.reshape(tf.transpose(tf.concat([rW_i, rW_f, rW_c, rW_o], axis=1), perm=[2, 0, 1]), shape=[input_size, -1]) # (1, 256*4, 512) -> (1, 1024, 512) -> (512, 1, 1024) -> (512, 1024)
         reverse_recurrent_kernel = tf.reshape(tf.transpose(tf.concat([rR_i, rR_f, rR_c, rR_o], axis=1), perm=[2, 0, 1]), shape=[hidden_size, -1]) # (1, 256*4, 256) -> (256, 1, 1024) -> (256, 1024)
         # backward
@@ -721,8 +863,14 @@ def make_node(
             bias_f=rB_f, # (1, 256)
             bias_c=rB_c, # (1, 256)
             bias_o=rB_o, # (1, 256)
+            peephole_i=rP_i, # (1, 256)
+            peephole_f=rP_f, # (1, 256)
+            peephole_o=rP_o, # (1, 256)
+            clip=clip,
+            input_forget=input_forget,
             is_bidirectional=False,
             go_backwards=True,
+            enable_rnn_unroll=enable_rnn_unroll,
         )
         output, hidden_state, cell_state = reverse_lstm(X, initial_state=backward_initial_state)
         output = tf.reverse(output, axis=[1])
@@ -736,9 +884,17 @@ def make_node(
         forward_bias_W = tf.reshape(tf.convert_to_tensor(B[0][:4*hidden_size]), shape=[4, hidden_size]) # TensorShape([4, 256])
         forward_bias_R = tf.reshape(tf.convert_to_tensor(B[0][4*hidden_size:4*hidden_size*2]), shape=[4, hidden_size]) # TensorShape([4, 256])
         forward_bias = forward_bias_W + forward_bias_R
+        forward_peepholes = None
+        if P is not None:
+            forward_peepholes = tf.reshape(tf.convert_to_tensor(P[0]), shape=[3, hidden_size])
         fW_i, fW_o, fW_f, fW_c = tf.split(value=forward_weight, num_or_size_splits=4, axis=0) # (1, 256, 512)
         fR_i, fR_o, fR_f, fR_c = tf.split(value=forward_recurrence_weight, num_or_size_splits=4, axis=0) # (1, 256, 256)
         fB_i, fB_o, fB_f, fB_c = tf.split(value=forward_bias, num_or_size_splits=4, axis=0) # (1, 256)
+        fP_i = None
+        fP_o = None
+        fP_f = None
+        if forward_peepholes is not None:
+            fP_i, fP_o, fP_f = tf.split(value=forward_peepholes, num_or_size_splits=3, axis=0) # (1, 256)
         forward_kernel = tf.reshape(tf.transpose(tf.concat([fW_i, fW_f, fW_c, fW_o], axis=1), perm=[2, 0, 1]), shape=[input_size, -1]) # (1, 256*4, 512) -> (1, 1024, 512) -> (512, 1, 1024) -> (512, 1024)
         forward_recurrent_kernel = tf.reshape(tf.transpose(tf.concat([fR_i, fR_f, fR_c, fR_o], axis=1), perm=[2, 0, 1]), shape=[hidden_size, -1]) # (1, 256*4, 256) -> (256, 1, 1024) -> (256, 1024)
 
@@ -747,9 +903,17 @@ def make_node(
         reverse_bias_W = tf.reshape(tf.convert_to_tensor(B[1][:4*hidden_size]), shape=[4, hidden_size]) # TensorShape([4, 256])
         reverse_bias_R = tf.reshape(tf.convert_to_tensor(B[1][4*hidden_size:4*hidden_size*2]), shape=[4, hidden_size]) # TensorShape([4, 256])
         reverse_bias = reverse_bias_W + reverse_bias_R
+        reverse_peepholes = None
+        if P is not None:
+            reverse_peepholes = tf.reshape(tf.convert_to_tensor(P[1]), shape=[3, hidden_size])
         rW_i, rW_o, rW_f, rW_c = tf.split(value=reverse_weight, num_or_size_splits=4, axis=0)
         rR_i, rR_o, rR_f, rR_c = tf.split(value=reverse_recurrence_weight, num_or_size_splits=4, axis=0)
         rB_i, rB_o, rB_f, rB_c = tf.split(value=reverse_bias, num_or_size_splits=4, axis=0)
+        rP_i = None
+        rP_o = None
+        rP_f = None
+        if reverse_peepholes is not None:
+            rP_i, rP_o, rP_f = tf.split(value=reverse_peepholes, num_or_size_splits=3, axis=0) # (1, 256)
         reverse_kernel = tf.reshape(tf.transpose(tf.concat([rW_i, rW_f, rW_c, rW_o], axis=1), perm=[2, 0, 1]), shape=[input_size, -1]) # (1, 256*4, 512) -> (1, 1024, 512) -> (512, 1, 1024) -> (512, 1024)
         reverse_recurrent_kernel = tf.reshape(tf.transpose(tf.concat([rR_i, rR_f, rR_c, rR_o], axis=1), perm=[2, 0, 1]), shape=[hidden_size, -1]) # (1, 256*4, 256) -> (256, 1, 1024) -> (256, 1024)
 
@@ -765,8 +929,14 @@ def make_node(
             bias_f=fB_f, # (1, 256)
             bias_c=fB_c, # (1, 256)
             bias_o=fB_o, # (1, 256)
+            peephole_i=fP_i, # (1, 256)
+            peephole_f=fP_f, # (1, 256)
+            peephole_o=fP_o, # (1, 256)
+            clip=clip,
+            input_forget=input_forget,
             is_bidirectional=True,
             go_backwards=False,
+            enable_rnn_unroll=enable_rnn_unroll,
         )
 
         # backward
@@ -781,8 +951,14 @@ def make_node(
             bias_f=rB_f, # (1, 256)
             bias_c=rB_c, # (1, 256)
             bias_o=rB_o, # (1, 256)
+            peephole_i=rP_i, # (1, 256)
+            peephole_f=rP_f, # (1, 256)
+            peephole_o=rP_o, # (1, 256)
+            clip=clip,
+            input_forget=input_forget,
             is_bidirectional=True,
             go_backwards=True,
+            enable_rnn_unroll=enable_rnn_unroll,
         )
         forward_output, forward_h, forward_c = \
             forward_lstm(X, initial_state=forward_initial_state) # [1, 24, 512], [[1, 256], [1, 256]] -> [1, 24, 256], [1, 256], [1, 256]
@@ -854,6 +1030,7 @@ def make_node(
                     'W': W,
                     'R': R,
                     'B': B,
+                    'hidden_size': hidden_size,
                     'sequence_lens': sequence_lens,
                     'initial_h': initial_h,
                     'initial_c': initial_c,
@@ -862,7 +1039,6 @@ def make_node(
                     'activation_alpha': tf_activation_alphas,
                     'activation_beta': tf_activation_betas,
                     'clip': clip,
-                    'hidden_size': hidden_size,
                     'input_forget': input_forget,
                     'layout': layout,
                 },
