@@ -65,12 +65,12 @@ def make_node(
         and before_op_output_shape_trans_4 \
         and before_op_output_shape_trans_5
 
-    input_tensor = get_constant_or_variable(
+    graph_node_input = get_constant_or_variable(
         graph_node.inputs[0],
         before_op_output_shape_trans,
     )
-    input_tensor = tf_layers_dict[input_tensor.name]['tf_node'] \
-        if isinstance(input_tensor, gs.Variable) else input_tensor
+    input_tensor = tf_layers_dict[graph_node_input.name]['tf_node'] \
+        if isinstance(graph_node_input, gs.Variable) else graph_node_input
 
     # Pre-process transpose
     input_tensor = pre_process_transpose(
@@ -81,7 +81,8 @@ def make_node(
     )
 
     input_tensor_shape = input_tensor.shape
-    input_tensor_rank = len(input_tensor_shape)
+    input_tensor_rank = len(input_tensor_shape) \
+        if input_tensor_shape != tf.TensorShape(None) else 1
 
     starts = None
     if len(graph_node.inputs) >= 2:
@@ -170,6 +171,9 @@ def make_node(
         'optype': graph_node.op,
         'shape': shape,
         'dtype': dtype,
+        'nhwc': tf_layers_dict[graph_node_input.name]['nhwc'] \
+            if isinstance(graph_node_input, gs.Variable) \
+                and 'nhwc' in tf_layers_dict[graph_node_input.name].keys() else False
     }
 
     # Param replacement - OP replacement
@@ -381,33 +385,53 @@ def make_node(
                 end_mask=end_mask_,
                 name=graph_node.name,
             )
-        # FlexStridedSlice generation suppression process
-        COMPRESSION_DEFAULT_VALUE = 5
-        onnx_slice_dims_count = 0
-        if isinstance(starts, np.ndarray):
-            onnx_slice_dims_count = len(starts)
-        elif hasattr(starts, 'numpy'):
-            onnx_slice_dims_count = len(starts.numpy())
-        elif isinstance(starts, int):
-            onnx_slice_dims_count = 1
-        else:
-            onnx_slice_dims_count = len(starts)
 
-        tf_layers_dict[graph_node_output.name]['tf_node'] = \
-            stridedslice_with_flexing_deterrence(
-                input_tensor=input_tensor,
-                begin=begin_,
-                end=end_,
-                strides=strides_,
-                begin_mask=begin_mask_,
-                end_mask=end_mask_,
-                ignore_axes=axes,
-                compression_defult_value=COMPRESSION_DEFAULT_VALUE,
-                onnx_slice_dims_count=onnx_slice_dims_count,
-                output_shape=tf_layers_dict[graph_node_output.name]['tf_node'].shape,
-                name=graph_node.name,
-                **kwargs,
-            )
+        check_input_shape = list(input_tensor_shape)
+        check_output_shape = list(tf_layers_dict[graph_node_output.name]['tf_node'].shape)
+        if None not in check_input_shape \
+            and None not in check_output_shape \
+            and check_input_shape == check_output_shape:
+            # Disable useless slice
+            tf_layers_dict[graph_node_output.name]['tf_node'] = \
+                tf.identity(
+                    input=input_tensor,
+                    name=graph_node.name,
+                )
+
+        elif input_tensor.shape != tf.TensorShape(None):
+            # FlexStridedSlice generation suppression process
+            COMPRESSION_DEFAULT_VALUE = 5
+            onnx_slice_dims_count = 0
+            if isinstance(starts, np.ndarray):
+                onnx_slice_dims_count = len(starts)
+            elif hasattr(starts, 'numpy'):
+                onnx_slice_dims_count = len(starts.numpy())
+            elif isinstance(starts, int):
+                onnx_slice_dims_count = 1
+            elif tf.keras.backend.is_keras_tensor(starts):
+                onnx_slice_dims_count = len(starts.shape)
+            else:
+                onnx_slice_dims_count = len(starts)
+
+            if onnx_slice_dims_count > COMPRESSION_DEFAULT_VALUE:
+                ignore_axes = axes
+                if axes is None:
+                    ignore_axes = [idx for idx in range(input_tensor_rank)]
+                tf_layers_dict[graph_node_output.name]['tf_node'] = \
+                    stridedslice_with_flexing_deterrence(
+                        input_tensor=input_tensor,
+                        begin=begin_,
+                        end=end_,
+                        strides=strides_,
+                        begin_mask=begin_mask_,
+                        end_mask=end_mask_,
+                        ignore_axes=ignore_axes,
+                        compression_defult_value=COMPRESSION_DEFAULT_VALUE,
+                        onnx_slice_dims_count=onnx_slice_dims_count,
+                        output_shape=tf_layers_dict[graph_node_output.name]['tf_node'].shape,
+                        name=graph_node.name,
+                        **kwargs,
+                    )
     else:
         # OP replacement
         tf_layers_dict[graph_node_output.name]['tf_node'] = \
